﻿using UnityEngine;
using System.Collections;

// Inherit from MonoBehavior and take another as T parameter
public class Singleton<T> : MonoBehaviour where T : MonoBehaviour
{
	// We’ll need to store the current instance of the T class
	private static T _instance;
	//We'll use this to know if the app is currently closing
	private static bool appIsClosing = false;

	// The public Instance of T, accessible from anywhere
	public static T Instance
	{
		// With an overridden getter method
		get
		{
			// If the app is closing...
			if (appIsClosing)
			{
				//... Return null, don't go further
				return null;
			}
			// If _instance is not assigned...
			if (_instance == null)
			{
				//... Find out if one already exists in the scene
				_instance = (T) FindObjectOfType(typeof(T));
				// If doesn't exist? Create it!
				if (_instance == null)
				{
					// Create a new GameObject...
					GameObject newSingleton = new GameObject();
					//... Add the T component to it
					_instance = newSingleton.AddComponent<T>();
					// Rename it with the T class's name
					newSingleton.name = typeof(T).ToString();
				}
				// Mark it as DontDestroyOnLoad
				DontDestroyOnLoad(_instance);
			}
			// Return the final _instance
			return _instance;
		}
	}

	// At start, for all singletons
	public void Start()
	{
		// Retrieve all instances of the singleton in the scene
		T[] allInstances = FindObjectsOfType(typeof(T)) as T[];
		
		// If there's more than one of them
		if(allInstances.Length > 1)
		{
			// For each of the found instances...
			foreach(T instanceToCheck in allInstances)
			{
				// If the found instance is not the current one
				if(instanceToCheck != Instance)
				{
					// Destroy it now
					Destroy(instanceToCheck.gameObject);
				}
			}
		}

		// Mark the existent instance as DontDestroyOnLoad
		DontDestroyOnLoad((T) FindObjectOfType(typeof(T)));
	}

	// When the application quits
	void OnApplicationQuit()
	{
		//... Set the appIsClosing boolean to true
		appIsClosing = true;
	}
}
