﻿using UnityEngine;
using System.Collections;

public class GameManager : Singleton<GameManager> {

	// We'll need our elemental switch UI stored here
	public GameObject elementalSwitchUI;
	// We'll need its attached FollowObject
	private FollowObject elementalFollowObject;
	// Declare the UIButton array
	UIButton[] allButtons;
	// Current power source with elemental switch UI
	public PowerSource currentPowerSource;

	// At Start
	void Start () {
		// Retrieve the elemental switch UI’s FolloObject
		elementalFollowObject =
			elementalSwitchUI.GetComponent<FollowObject>();
		// Hide elemental switch UI
		elementalSwitchUI.SetActive(false);
		// Retrieve all elemental buttons
		allButtons =
			elementalSwitchUI.GetComponentsInChildren<UIButton>(true);
	}

	// Method called when pause button is pressed
	public void PausePressed()
	{
		// Request pause
		SetPause(true);
	}
	
	// Method called when resume button is pressed
	public void ResumePresssed()
	{
		// Request unpause
		SetPause(false);
	}
	
	// Method called when Exit button is pressed
	public void ExitPressed()
	{
		// Request to return to main menu
		ReturnToMenu();
	}

	// Use this method to return to the main menu
	public void ReturnToMenu()
	{
		// Unpause the game
		ResumePresssed();
		// Destroy the player instance
		Destroy(PlayerController.Instance);
		
		Transform mainMenu = null;
		mainMenu = MenuManager.Instance.transform.FindChild("Main");
		if(mainMenu != null)
		{
			// Set the MenuManager | Main scale to 1
			mainMenu.localScale = Vector3.one;
		}
		// Launch the EnterMenu couroutine
		StartCoroutine(EnterMenu());
	}

	// EnterMenu Coroutine
	private IEnumerator EnterMenu()
	{
		// Show the main menu UI by enabling cam
		MenuManager.Instance.uiCamera.enabled = true;
		// Fade in the UI background
		TweenAlpha.Begin(MenuManager.Instance.background, 0.5f, 1);
		// Wait for the tween to finish
		yield return new WaitForSeconds(0.5f);
		// Load the Menu scene now
		Application.LoadLevel("Menu");  
	}

	// Use this method to pause / unpause game
	public void SetPause(bool state)
	{
		// Set the timescale to appropriate value
		Time.timeScale = (state ? 0 : 1);
	}

	// Show or hide elemental switch UI
	public void ShowElementalSwitch(Transform targetObject)
	{
		// If the target object to follow isn't null 
		if(targetObject != null)
		{
			// Set the elements switch's current power source
			currentPowerSource = targetObject.GetComponent<PowerSource>();
			// Enable the elemental switch UI
			elementalSwitchUI.SetActive(true);
			// Make sure it follows the target
			elementalFollowObject.target = targetObject;
			// Set its scale to {0.01,0.01,0.01}
			elementalSwitchUI.transform.localScale =
				new Vector3(0.01f,0.01f,0.01f);
			// Start a scale tween on it
			TweenScale.Begin(elementalSwitchUI, 0.2f, Vector3.one);
		}
		// If the target object is null
		else
		{
			// Disable the elemental switch UI
			elementalSwitchUI.SetActive (false);
		}
	}

	// Execute when Fire button is pressed
	public void FirePressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Fire);
	}
	
	// Execute when Ice button is pressed
	public void IcePressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Ice);
	}
	
	// Execute when Lightning button is pressed
	public void LightningPressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Lightning);
	}
	
	// Execute when Water button is pressed
	public void WaterPressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Water);
	}

	// Enable all elemental buttons
	public void EnableAllButtons()
	{
		// For each elemental button
		foreach(UIButton currentButton in allButtons)
		{
			// Mark it as enabled
			currentButton.isEnabled = true;
			// Update its color
			currentButton.UpdateColor(true);
		}
	}

	// Used to enable / disable elemental buttons
	public void SetButtonState(Elements.Type type, bool state)
	{
		// For each button in allButtons[]
		foreach (UIButton currentButton in allButtons)
		{
			// If currentButton is the one to enable/disable
			if (currentButton.name == type.ToString())
			{
				// Change its isEnabled state
				currentButton.isEnabled = state;
				// Update its color now
				currentButton.UpdateColor(true);
				return;
			}
		}
	}

	// EXISTING METHODS

	public Transform GetElementalSwitchUITarget()
	{
		return elementalFollowObject.target;
	}
}
