﻿using UnityEngine;
using System.Collections;

public class GameManager : Singleton<GameManager> {

	// We'll need our elemental switch UI stored here
	public GameObject elementalSwitchUI;
	// We'll need its attached FollowObject
	private FollowObject elementalFollowObject;
	// Declare the UIButton array
	UIButton[] allButtons;
	// Current power source with elemental switch UI
	public PowerSource currentPowerSource;
	// We need the HoldFeedback filled sprite
	public UISprite holdFeedback;
	// We need the HoldFeedback's FollowObject
	private FollowObject holdFeedbackFollowObject;
	// Game Main Camera
	Camera gameMainCamera;
	// 3D UI Camera
	Camera inGame3DUICamera;

	// At Start
	void Start () {
		// Retrieve the elemental switch UI’s FolloObject
		elementalFollowObject =
			elementalSwitchUI.GetComponent<FollowObject>();
		// Hide elemental switch UI
		elementalSwitchUI.SetActive(false);
		// Retrieve all elemental buttons
		allButtons =
			elementalSwitchUI.GetComponentsInChildren<UIButton>(true);
		// Retrieve the HoldFeedback's FolloObject
		holdFeedbackFollowObject =
			holdFeedback.GetComponent<FollowObject>();
		// Hide the HoldFeedback sprite
		holdFeedback.enabled = false;
		// Retrieve the game's Main Camera
		gameMainCamera = Camera.main;
		// Retrieve the 3D UI Camera
		inGame3DUICamera = NGUITools.FindCameraForLayer(10);
	}

	// Routine that displays the touch hold feedback
	public IEnumerator HoldFeedbackRoutine(object[] parameters)
	{
		// Current fill amount to set
		float currentFillAmount = 0;
		// Retrieve concerned object's transform
		Transform concernedObject = parameters[0] as Transform;
		// The touch hold duration
		float holdDuration = (float)(parameters[1]);
		// Delay before the hold process starts
		float delay = 0.2f;
		// Retrieve concerned object's CustomizableUIElement
		CustomizableUIElement customizableElement =
			concernedObject.GetComponent<CustomizableUIElement>();
		
		// Retrieve concerned object's ApproachOnClick
		ApproachOnClick approachOnClick =
			concernedObject.GetComponent<ApproachOnClick>();
		// Retrieve concerned object's PowerSource
		PowerSource powerSource =
			concernedObject.GetComponent<PowerSource>();
		
		// Set filled sprite's fill amount to zero
		holdFeedback.fillAmount = 0;
		// Show the hold feedback sprite
		holdFeedback.enabled = true;
		
		// Make the hold feedback follow the power source
		holdFeedbackFollowObject.target = concernedObject;
		
		// Wait before starting sprite filling process
		yield return new WaitForSeconds(delay);
		
		// Make sure the release doesn't move the player
		if(approachOnClick != null)
		{
			approachOnClick.CancelMovementRequest();
		}
		
		// While holdFeedback sprite isn't filled...
		while(currentFillAmount< 1)
		{
			// Increase the currentFillAmount
			currentFillAmount += (Time.deltaTime / holdDuration);
			// Update the holdFeedback's filled sprite
			holdFeedback.fillAmount = currentFillAmount;
			// Wait for next frame
			yield return null;
		}
		
		/* ...Once the currentFillAmount>= 1
	   Hide the holdFeedback sprite */
		holdFeedback.enabled = false;
		
		// If the hold concerns a power source
		if(powerSource != null)
		{
			// Show the elemental switch UI now
			ShowElementalSwitch(concernedObject);
		}
		// If it's a customizable UI element
		else if(customizableElement != null)
		{
			// Make it draggable
			customizableElement.SetDraggable(true);
		}
	}

	// HoldFeedback either follows env 3D object or 3D UI
	public void HoldFollowsEnvironment(bool follows3DObject)
	{
		// If HoldFeedback must follow a 3D env object
		if(follows3DObject)
		{
			// Set the FollowObject's camera to the env cam
			holdFeedbackFollowObject.mainCamera = gameMainCamera;
		}
		// If it must follow a 3D UI widget
		else
		{
			// Set the FollowObject's camera to the 3DUI cam
			holdFeedbackFollowObject.mainCamera = inGame3DUICamera;
		}
	}

	// Method that cancels the hold process
	public void CancelHoldFeedback()
	{
		// Stop the coroutine
		StopCoroutine("HoldFeedbackRoutine");
		// Hide the holdFeedback sprite
		holdFeedback.enabled = false;
	}

	// Method called when pause button is pressed
	public void PausePressed()
	{
		// Request pause
		SetPause(true);
	}
	
	// Method called when resume button is pressed
	public void ResumePresssed()
	{
		// Request unpause
		SetPause(false);
	}
	
	// Method called when Exit button is pressed
	public void ExitPressed()
	{
		// Request to return to main menu
		ReturnToMenu();
	}

	// Use this method to return to the main menu
	public void ReturnToMenu()
	{
		// Unpause the game
		ResumePresssed();
		// Destroy the player instance
		Destroy(PlayerController.Instance);
		
		Transform mainMenu = null;
		mainMenu = MenuManager.Instance.transform.FindChild("Main");
		if(mainMenu != null)
		{
			// Set the MenuManager | Main scale to 1
			mainMenu.localScale = Vector3.one;
		}
		// Launch the EnterMenu couroutine
		StartCoroutine(EnterMenu());
	}

	// EnterMenu Coroutine
	private IEnumerator EnterMenu()
	{
		// Show the main menu UI by enabling cam
		MenuManager.Instance.uiCamera.enabled = true;
		// Fade in the UI background
		TweenAlpha.Begin(MenuManager.Instance.background, 0.5f, 1);
		// Wait for the tween to finish
		yield return new WaitForSeconds(0.5f);
		// Load the Menu scene now
		Application.LoadLevel("Menu");
		// Enable all objects in enableAtAwake array
		foreach(GameObject currentGO in MenuManager.Instance.enableAtAwake)
		{
			currentGO.SetActive(true);
		}
		// Destroy the GameManager on game exit
		Destroy(this.gameObject);
	}

	// Use this method to pause / unpause game
	public void SetPause(bool state)
	{
		// Set the timescale to appropriate value
		Time.timeScale = (state ? 0 : 1);
	}

	// Show or hide elemental switch UI
	public void ShowElementalSwitch(Transform targetObject)
	{
		// If the target object to follow isn't null 
		if(targetObject != null)
		{
			// Set the elements switch's current power source
			currentPowerSource = targetObject.GetComponent<PowerSource>();
			// Enable the elemental switch UI
			elementalSwitchUI.SetActive(true);
			// Make sure it follows the target
			elementalFollowObject.target = targetObject;
			// Set its scale to {0.01,0.01,0.01}
			elementalSwitchUI.transform.localScale =
				new Vector3(0.01f,0.01f,0.01f);
			// Start a scale tween on it
			TweenScale.Begin(elementalSwitchUI, 0.2f, Vector3.one);
		}
		// If the target object is null
		else
		{
			// Disable the elemental switch UI
			elementalSwitchUI.SetActive (false);
		}
	}

	// Execute when Fire button is pressed
	public void FirePressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Fire);
	}
	
	// Execute when Ice button is pressed
	public void IcePressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Ice);
	}
	
	// Execute when Lightning button is pressed
	public void LightningPressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Lightning);
	}
	
	// Execute when Water button is pressed
	public void WaterPressed()
	{
		// Request the power source to switch element;
		currentPowerSource.SwitchElement(Elements.Type.Water);
	}

	// Enable all elemental buttons
	public void EnableAllButtons()
	{
		// For each elemental button
		foreach(UIButton currentButton in allButtons)
		{
			// Mark it as enabled
			currentButton.isEnabled = true;
			// Update its color
			currentButton.UpdateColor(true);
		}
	}

	// Used to enable / disable elemental buttons
	public void SetButtonState(Elements.Type type, bool state)
	{
		// For each button in allButtons[]
		foreach (UIButton currentButton in allButtons)
		{
			// If currentButton is the one to enable/disable
			if (currentButton.name == type.ToString())
			{
				// Change its isEnabled state
				currentButton.isEnabled = state;
				// Update its color now
				currentButton.UpdateColor(true);
				return;
			}
		}
	}

	// EXISTING METHODS

	public Transform GetElementalSwitchUITarget()
	{
		return elementalFollowObject.target;
	}
}
