﻿using UnityEngine;
using System.Collections;

public class PowerSource : MonoBehaviour {

	// EXISTING VARIABLES

	// This power source's current element type
	public Elements.Type type;
	// The current energy of this power source
	private float energy = 1;
	// The elemental type switch duration in seconds
	public float switchDuration = 4f;
	// Bool to define if the power source is ready
	private bool available;
	// We need to store the 2D UI Root
	private GameObject uiRoot2D;
	// We need to store the switch slider
	private UISlider progressSlider;
	// We'll need the UIDragObject component
	private UIDragObject dragObject;
	// We'll need the source's associated light
	private Light light;
	// We'll need the mesh renderer
	private MeshRenderer mesh;
	// We'll need to store the unavailable transparent material
	private Material unavailableMat;

	// If the power source triggers the OnPress event
	void OnPress(bool pressed)
	{
		// If the power source is pressed
		if(pressed)
		{
			// Re-enable all buttons
			GameManager.Instance.EnableAllButtons();
			// Disable this power source's element button
			GameManager.Instance.SetButtonState(type, false);
			// Request to hide the elemental switch
			GameManager.Instance.ShowElementalSwitch(null);

			//If in editor 
			/*if(Application.isEditor)
			{
				// If it's a right click
				if(UICamera.currentTouchID == -2 && available)
				{
					// Request to display the elemental switch
					GameManager.Instance.ShowElementalSwitch(transform);
				}
			}
			// If we're on the final build
			else
			{*/
				// If it's a one-finger touch
				if(UICamera.currentTouchID == 0 && available)
				{
					// Parameters to pass to the Coroutine
					object[] parameters = new object[2]{transform, 0.5f};
					// Cancel hold feedback if there is one
					GameManager.Instance.CancelHoldFeedback();
					// Set the HoldFeedback to follow 3D Object
					GameManager.Instance.HoldFollowsEnvironment(true);
					// Start the HoldFeedback
					GameManager.Instance.StartCoroutine("HoldFeedbackRoutine", parameters);
				}
			//}
		}
		// If power source is released
		else
		{
			// Cancel hold feedback if there is one
			GameManager.Instance.CancelHoldFeedback();
		}
	}

	// If the power source is dragged
	void OnDrag(Vector2 delta)
	{
		// Cancel the hold feedback process
		GameManager.Instance.CancelHoldFeedback();
	}


	// Method to create the progress slider
	private void CreateProgressSlider()
	{
		// Instantiate the new progress slider
		GameObject progressObject;
		progressObject =  NGUITools.AddChild(uiRoot2D, 
		                                     Resources.Load("Prefabs/Progress") as GameObject);
		// Retrieve its attached UISlider component
		progressSlider = progressObject.GetComponent<UISlider>();
		// Add a FollowObject to the slider
		progressSlider.gameObject.AddComponent(typeof(FollowObject));
		// Retrieve and store it                        
		FollowObject sliderFollowObject =
			progressSlider.GetComponent<FollowObject>();
		// Configure it to follow this power source
		sliderFollowObject.target = transform;
		sliderFollowObject.mainCamera = 
			GameObject.Find("GameCamera").camera;
		sliderFollowObject.uiCamera = 
			NGUITools.FindCameraForLayer(9);
		sliderFollowObject.offset = new Vector3(0,0.25f,0);
	}

	// EXISTING METHODS

	// Method used to switch element to a new one
	public void SwitchElement(Elements.Type newElement)
	{
		// Set unavailable transparent material
		mesh.material = unavailableMat;
		// Start the coroutine to switch element
		StartCoroutine(SwitchElementRoutine(newElement));
	}

	// The coroutine to switch power source type
	private IEnumerator SwitchElementRoutine(Elements.Type newElement)
	{
		//Hide elemental switch menu
		GameManager.Instance.ShowElementalSwitch(null);
		// Set the energy to zero
		energy = 0;
		// Make it unavailable while it's charging
		available = false;
		// Make it non-draggable while charging
		dragObject.enabled = false;
		// Disable associated light
		light.enabled = false;
		// Create the new slider
		CreateProgressSlider();
		// At each frame, while energy is not full
		while (energy < 1)
		{
			// Add a little energy
			energy += (Time.deltaTime / switchDuration);
			// Update associated progress slider
			progressSlider.value = energy;
			// Wait until next frame
			yield return null;
		}
		// When finished charging, make sure it's set to 1
		energy = 1;
		// Set the power source type to the new one
		SetNewElement(newElement);
		// Make it available again
		available = true;
		// Make it draggable again
		dragObject.enabled = true;
		// Re-enable the light
		light.enabled = true;
		// Destroy the slider
		Destroy (progressSlider.gameObject);
	}

	// Set a new source element type now
	private void SetNewElement(Elements.Type newElement)
	{
		// Update the type with the requested element
		type = newElement;
		// Declare the new light color
		Color32 newLightColor;
		// The new material's path
		string newMatPath = "3D/Materials/" + newElement.ToString();
		// Assign the new material
		mesh.material = Resources.Load(newMatPath) as Material;

		// Configure color depending on new element
		switch(newElement)
		{
		case Elements.Type.Fire:
			newLightColor = new Color32(255, 140, 90, 255);
			break;
		case Elements.Type.Ice:
			newLightColor = new Color32(230, 255, 250, 255);
			break;
		case Elements.Type.Lightning:
			newLightColor = new Color32(40, 255, 255, 255);
			break;
		// If it's water, or anything else
		default:
			newLightColor = new Color32(0, 50, 255, 255);
			break;
		}

		// Assign the new light color
		light.color = newLightColor;
	}

	// Upon interaction, set the player to take the element
	public void Interact()
	{
		PlayerController.Instance.SetNewElement(type);
	}
	

	// Initialize values at start
	private void Start()
	{
		// Set energy at 1
		energy = 1f;
		// Make it available
		available = true;
		// Retreive the InGame2DUI root
		uiRoot2D = GameObject.Find("InGame2DUI");
		// Retrieve the UIDragObject
		dragObject = GetComponent<UIDragObject>();
		// Retrieve associated  light
		light = GetComponentInChildren<Light>();
		// Retrieve the mesh renderer
		mesh = GetComponentInChildren<MeshRenderer>();
		// Load the unavailable transparent material
		unavailableMat = Resources.Load("3D/Materials/Unavailable") as Material;
		// Set the default element
		SetNewElement(type);
	}

}
